@@ -3,6 +3,7 @@
 import logging
 import re
 import urllib
+import urlparse
 import types
 from itertools import izip
 from dispatch import generic, strategy, functions
@@ -393,7 +394,15 @@ def _execute_func(func, template, format, content_type, mapping, fragment, args,
         tg_util.remove_keys(kw, ["tg_random", "tg_format"]
             + config.get("tg.ignore_parameters", []))
     else:
+        # get special parameters used by upstream decorators like paginate
+        try:
+            tg_kw = dict([(k, v) for k, v in kw.items() if k in func._tg_args])
+        except AttributeError:
+            tg_kw = {}
+        # remove excessive parameters
         args, kw = tg_util.adapt_call(func, args, kw)
+        # add special parameters again
+        kw.update(tg_kw)
     if config.get('server.environment', 'development') == 'development':
         # Only output this in development mode: If it's a field storage object,
         # this means big memory usage, and we don't want that in production
@@ -501,9 +510,9 @@ def url(tgpath, tgparams=None, **kw):
     """Computes URLs.
 
     tgpath can be a list or a string. If the path is absolute (starts
-    with a "/"), the server.webpath and the approot of the application
-    are prepended to the path. In order for the approot to be
-    detected properly, the root object should extend
+    with a "/"), the server.webpath, SCRIPT_NAME and the approot of the
+    application are prepended to the path. In order for the approot to
+    be detected properly, the root object should extend
     controllers.RootController.
 
     Query parameters for the URL can be passed in as a dictionary in
@@ -514,14 +523,17 @@ def url(tgpath, tgparams=None, **kw):
 
     """
     if not isinstance(tgpath, basestring):
-        tgpath = "/".join(list(tgpath))
-    if tgpath.startswith("/"):
+        tgpath = '/'.join(list(tgpath))
+    if tgpath.startswith('/'):
+        webpath = (config.get('server.webpath') or '').rstrip('/')
         if tg_util.request_available():
             check_app_root()
             tgpath = request.app_root + tgpath
-        result = config.get("server.webpath", "") + tgpath
-    else:
-        result = tgpath
+            try:
+                webpath += request.wsgi_environ['SCRIPT_NAME'].rstrip('/')
+            except (AttributeError, KeyError):
+                pass
+        tgpath = webpath + tgpath
     if tgparams is None:
         tgparams = kw
     else:
@@ -542,16 +554,20 @@ def url(tgpath, tgparams=None, **kw):
             if v is None:
                 continue
             if isinstance(v, unicode):
-                v = v.encode("utf8")
-            args.append("%s=%s" % (k, urllib.quote(str(v))))
+                v = v.encode('utf8')
+            args.append((k, str(v)))
     if args:
-        result += "?" + "&".join(args)
-    return result
+        query_string = urllib.urlencode(args, True)
+        if '?' in tgpath:
+            tgpath += '&' + query_string
+        else:
+            tgpath += '?' + query_string
+    return tgpath
 
 
 def check_app_root():
     """Sets request.app_root if needed."""
-    if hasattr(request, "app_root"):
+    if hasattr(request, 'app_root'):
         return
     found_root = False
     trail = request.object_trail
@@ -571,10 +587,10 @@ def check_app_root():
             found_root = True
         if found_root and i > 0:
             rootlist.insert(0, path)
-    app_root = "/".join(rootlist)
-    if not app_root.startswith("/"):
-        app_root = "/" + app_root
-    if app_root.endswith("/"):
+    app_root = '/'.join(rootlist)
+    if not app_root.startswith('/'):
+        app_root = '/' + app_root
+    if app_root.endswith('/'):
         app_root = app_root[:-1]
     request.app_root = app_root
 
@@ -586,10 +602,27 @@ def redirect(redirect_path, redirect_params=None, **kw):
     users to both call it as a function or to raise it as an exception.
 
     """
+    if not isinstance(redirect_path, basestring):
+        redirect_path = '/'.join(list(redirect_path))
+    if not redirect_path.startswith('/'):
+        path = request.path_info
+        check_app_root()
+        if path.startswith(request.app_root):
+            path = path[len(request.app_root):]
+        redirect_path = urlparse.urljoin(path, redirect_path)
     raise cherrypy.HTTPRedirect(url(tgpath=redirect_path,
         tgparams=redirect_params, **kw))
 
 
-__all__ = ["expose", "validate", "redirect", "flash",
-           "Root", "RootController", "Controller",
-           "error_handler", "exception_handler"]
+__all__ = [
+    "Controller",
+    "error_handler",
+    "exception_handler",
+    "expose",
+    "flash",
+    "redirect",
+    "Root",
+    "RootController",
+    "url",
+    "validate",
+]
