@@ -10,10 +10,17 @@ def set_identity_errors(errors):
         errors = [errors]
     cherrypy.request.identity_errors = list(errors)
 
-
 def get_identity_errors():
     return getattr(cherrypy.request, 'identity_errors', [])
 
+def get_failure_url(errors=None):
+    url = turbogears.config.get('identity.failure_url', None)
+    if url is None:
+        msg = "Missing URL for identity failure. Please fix this in app.cfg."
+        raise IdentityConfigurationException(msg)
+    if callable(url):
+        url = url(errors)
+    return url
 
 class IdentityException(Exception):
     """Base class for all Identity exceptions."""
@@ -70,20 +77,14 @@ class IdentityFailure(cherrypy.InternalRedirect, IdentityException):
     def __init__(self, errors):
         """Setup identity errors on the request and get URL from config."""
         set_identity_errors(errors)
-        url = turbogears.config.get('identity.failure_url', None)
-        if url is None:
-            msg = "Missing URL for identity failure. Please fix this in app.cfg"
-            raise IdentityConfigurationException(msg)
-        if callable(url):
-            url = url(errors)
-        force_external = turbogears.config.get(
-                "identity.force_external_redirect", False)
-        if force_external:
-            # We need to use external redirect for https since
-            # we are managed by Apache/nginx or something else
-            # that CherryPy won't find.
+        url = get_failure_url(errors)
+        if turbogears.config.get('identity.force_external_redirect', False):
+            # We need to use external redirect for https since we are managed
+            # by Apache/nginx or something else that CherryPy won't find.
+            # We also need to set the forward_url, because the Referer header
+            # won't work with an external redirect.
             params = cherrypy.request.params
-            params['forward_url'] = cherrypy.request.path
+            params['forward_url'] = cherrypy.request.path_info
             raise cherrypy.HTTPRedirect(turbogears.url(url, params))
         else:
             # use internal redirect which is quicker
