@@ -232,6 +232,18 @@ class MyRoot(controllers.RootController):
     def internal_redirect_target(self, **kwargs):
         return "redirected OK"
 
+    [expose()]
+    def redirect_to_path_str(self, path):
+        raise redirect(path + '/index')
+
+    [expose()]
+    def redirect_to_path_list(self, path):
+        raise redirect([path, 'index'])
+
+    [expose()]
+    def redirect_to_path_tuple(self, path):
+        raise redirect((path, 'index'))
+
 
 class TestRoot(unittest.TestCase):
 
@@ -666,6 +678,15 @@ class TestURLs(unittest.TestCase):
         except cherrypy.HTTPRedirect, e:
             assert "http://localhost/coolsite/root/subthing/foo" in e.urls
 
+    def test_redirect_to_path(self):
+        for path_type in ('str', 'list', 'tuple'):
+            for path in ('subthing', '/subthing'):
+                url = "/redirect_to_path_%s?path=%s" % (path_type, path)
+                testutil.create_request(url)
+                assert cherrypy.response.status.startswith("302"), url
+                assert (cherrypy.response.headers['Location']
+                    == 'http://localhost/subthing/index'), url
+
     def test_multi_values(self):
         testutil.create_request("/")
         assert url("/foo", bar=[1, 2]) in \
@@ -706,25 +727,25 @@ class TestURLs(unittest.TestCase):
         assert 'foo' in url('/foo', params, spamm='ham')
         assert params['spamm'] == 'eggs'
 
-def test_index_trailing_slash():
-    """If there is no trailing slash on an index method call, redirect"""
-    cherrypy.root = SubApp()
-    cherrypy.root.foo = SubApp()
-    testutil.create_request("/foo")
-    assert cherrypy.response.status.startswith("302")
+    def test_index_trailing_slash(self):
+        """If there is no trailing slash on an index method call, redirect"""
+        cherrypy.root = SubApp()
+        cherrypy.root.foo = SubApp()
+        testutil.create_request("/foo")
+        assert cherrypy.response.status.startswith("302")
 
-def test_can_use_internally_defined_arguments():
-    """Can use argument names that are internally used by TG in controllers"""
+    def test_can_use_internally_defined_arguments(self):
+        """Can use argument names that are internally used by TG in controllers"""
 
-    class App(controllers.RootController):
+        class App(controllers.RootController):
 
-        [expose()]
-        def index(self, **kw):
-            return "\n".join(["%s:%s" % i for i in kw.iteritems()])
+            [expose()]
+            def index(self, **kw):
+                return "\n".join(["%s:%s" % i for i in kw.iteritems()])
 
-    cherrypy.root = App()
-    testutil.create_request("/?format=foo&template=bar&fragment=boo")
-    output = cherrypy.response.body[0]
-    assert "format:foo" in output
-    assert "template:bar" in output
-    assert "fragment:boo" in output
+        cherrypy.root = App()
+        testutil.create_request("/?format=foo&template=bar&fragment=boo")
+        output = cherrypy.response.body[0]
+        assert "format:foo" in output
+        assert "template:bar" in output
+        assert "fragment:boo" in output
