@@ -24,8 +24,7 @@ query_methods = 'Q QA SR SO SL'.split()
 if SASelectResults is None:
     query_methods.remove('SR')
 
-config.update({"tg.strict_parameters": True,
-               "sqlalchemy.dburi": "sqlite:///:memory:"})
+config.update({"sqlalchemy.dburi": "sqlite:///:memory:"})
 
 # sqlalchemy setup
 sqlalchemy_cleanup()
@@ -36,6 +35,19 @@ database.set_db_uri("sqlite:///:memory:")
 __connection__ = hub = database.PackageHub('test_paginate')
 
 
+class listlike(object):
+    """A minimum list-like object that is needed for paginate."""
+
+    def __init__(self, *args):
+        self._list = list(*args)
+
+    def __len__(self):
+        return len(self._list)
+
+    def __getitem__(self, key):
+        return self._list[key]
+
+
 def test_sort_ordering():
     """Test the sort_ordering auxiliary function."""
     s = []
@@ -99,6 +111,7 @@ def test_sort_in_memory():
             assert isinstance(ab[0], dict) and ab[0].get('a', 'b') == ['a'] \
                 and getattr(ab[0], 'a', 'b') == 'b'
         assert sort_data(ab, []) == ab
+        assert sort_data(ab, ['b'], False) == ab
         assert sort_data(ab, ['a']) == ab
         assert sort_data(ab, ['a', 'b']) == ab
         assert sort_data(ab, ['a', '-b']) == aB
@@ -262,9 +275,13 @@ class TestSpy(unittest.TestCase):
 class TestPagination(unittest.TestCase):
     """Base class for all Paginate TestCases"""
 
+    def setUp(self):
+        cherrypy.root = self.MyRoot()
+
     def request(self, url):
         create_request(url)
         self.body = cherrypy.response.body[0]
+        self.status = cherrypy.response.status
         if "fail: " in self.body:
             print self.body
             assert False, "Spy alert! Check body output for details..."
@@ -283,12 +300,27 @@ class TestBasicPagination(TestPagination):
             return dict(data=data, spy=spy)
 
         [expose()]
-        [paginate("data", limit=4, allow_limit_override=True)]
+        [paginate("data", limit=4)]
+        def listlike(self):
+            spy = Spy(var_name='data', pages=xrange(1, 4), limit=4, page_count=3,
+                      order=None, ordering=[], row_count=10)
+            data = listlike(range(10))
+            return dict(data=data, spy=spy)
+
+        [expose()]
+        [paginate("data", limit=4, max_limit=None)]
         def custom_limit(self):
             spy = Spy(var_name='data', order=None, ordering=[], row_count=10)
             data = range(10)
             return dict(data=data, spy=spy)
 
+        [expose()]
+        [paginate("data", limit=4, max_limit=7)]
+        def max_limit(self):
+            spy = Spy(var_name='data', order=None, ordering=[], row_count=10)
+            data = range(10)
+            return dict(data=data, spy=spy)
+
         [expose()]
         [paginate("data")]
         def default_max_pages(self):
@@ -320,7 +352,7 @@ class TestBasicPagination(TestPagination):
             return dict(data=data)
 
         [expose()]
-        [paginate("data", limit=4, dynamic_limit='foobar', allow_limit_override=True)]
+        [paginate("data", limit=4, dynamic_limit='foobar', max_limit=None)]
         def dynamic(self):
             spy = Spy(var_name='data', pages=xrange(1, 3), limit=7, page_count=2,
                       order=None, ordering=[], row_count=10)
@@ -341,7 +373,7 @@ class TestBasicPagination(TestPagination):
             return dict(foo=foo, bar=bar, spy_foo=spy_foo, spy_bar=spy_bar)
 
         [expose()]
-        [paginate("data", limit=4, allow_limit_override=True)]
+        [paginate("data", limit=4, max_limit=None)]
         def empty(self):
             spy = Spy(var_name='data', pages=xrange(1, 1), page_count=0,
                       order=None, ordering=[], row_count=0,
@@ -350,9 +382,6 @@ class TestBasicPagination(TestPagination):
             return dict(data=data, spy=spy)
 
 
-    def setUp(self):
-        cherrypy.root = self.MyRoot()
-
     def test_pagination_old_style(self):
         self.request("/basic")
         assert '"data": [0, 1, 2, 3]' in self.body
@@ -360,72 +389,93 @@ class TestBasicPagination(TestPagination):
         Spy.assert_ok(self.body, 'first_item', 1)
         Spy.assert_ok(self.body, 'last_item', 4)
 
-        self.request("/basic/?tg_paginate_no=1")
+        self.request("/basic?tg_paginate_no=1")
         assert '"data": [0, 1, 2, 3]' in self.body
         Spy.assert_ok(self.body, 'current_page', 1)
         Spy.assert_ok(self.body, 'first_item', 1)
         Spy.assert_ok(self.body, 'last_item', 4)
 
-        self.request("/basic/?tg_paginate_no=2")
+        self.request("/basic?tg_paginate_no=2")
         assert '"data": [4, 5, 6, 7]' in self.body
         Spy.assert_ok(self.body, 'current_page', 2)
         Spy.assert_ok(self.body, 'first_item', 5)
         Spy.assert_ok(self.body, 'last_item', 8)
 
-        self.request("/basic/?tg_paginate_no=3")
+        self.request("/basic?tg_paginate_no=3")
         assert '"data": [8, 9]' in self.body
         Spy.assert_ok(self.body, 'current_page', 3)
         Spy.assert_ok(self.body, 'first_item', 9)
         Spy.assert_ok(self.body, 'last_item', 10)
 
     def test_pagination_new_style(self):
-        self.request("/basic/?data_tgp_no=1")
+        self.request("/basic?data_tgp_no=1")
         assert '"data": [0, 1, 2, 3]' in self.body
         Spy.assert_ok(self.body, 'current_page', 1)
         Spy.assert_ok(self.body, 'first_item', 1)
         Spy.assert_ok(self.body, 'last_item', 4)
 
-        self.request("/basic/?data_tgp_no=2")
+        self.request("/basic?data_tgp_no=2")
         assert '"data": [4, 5, 6, 7]' in self.body
         Spy.assert_ok(self.body, 'current_page', 2)
         Spy.assert_ok(self.body, 'first_item', 5)
         Spy.assert_ok(self.body, 'last_item', 8)
 
-        self.request("/basic/?data_tgp_no=3")
+        self.request("/basic?data_tgp_no=3")
         assert '"data": [8, 9]' in self.body
         Spy.assert_ok(self.body, 'current_page', 3)
         Spy.assert_ok(self.body, 'first_item', 9)
         Spy.assert_ok(self.body, 'last_item', 10)
 
+    def test_pagination_listlike(self):
+        self.request("/listlike?data_tgp_no=2")
+        assert '"data": [4, 5, 6, 7]' in self.body
+        Spy.assert_ok(self.body, 'current_page', 2)
+        Spy.assert_ok(self.body, 'first_item', 5)
+        Spy.assert_ok(self.body, 'last_item', 8)
+
     def test_limit_override(self):
         # can't override limit (old style)
-        self.request('/basic/?tg_paginate_limit=2')
+        self.request('/basic?tg_paginate_limit=2')
         assert '"data": [0, 1, 2, 3]' in self.body
 
         # can't override limit (new style)
-        self.request("/basic/?data_tgp_limit=2")
+        self.request("/basic?data_tgp_limit=2")
         assert '"data": [0, 1, 2, 3]' in self.body
 
+        # can't override limit with upper bound
+        self.request("/max_limit?data_tgp_limit=9")
+        assert '"data": [0, 1, 2, 3, 4, 5, 6]' in self.body
+        Spy.assert_ok(self.body, 'page_count', 2)
+        Spy.assert_ok(self.body, 'limit', 7)
+        Spy.assert_ok(self.body, 'pages', xrange(1, 3))
+
         # can override limit (old style)
-        self.request('/custom_limit/?tg_paginate_limit=2')
+        self.request('/custom_limit?tg_paginate_limit=2')
         assert '"data": [0, 1]' in self.body
         Spy.assert_ok(self.body, 'page_count', 5)
         Spy.assert_ok(self.body, 'limit', 2)
         Spy.assert_ok(self.body, 'pages', xrange(1, 6))
 
         # can override limit (new style)
-        self.request("/custom_limit/?data_tgp_limit=2")
+        self.request("/custom_limit?data_tgp_limit=2")
         assert '"data": [0, 1]' in self.body
         Spy.assert_ok(self.body, 'page_count', 5)
         Spy.assert_ok(self.body, 'limit', 2)
         Spy.assert_ok(self.body, 'pages', xrange(1, 6))
 
+        # can override limit with upper bound
+        self.request("/max_limit?data_tgp_limit=5")
+        assert '"data": [0, 1, 2, 3, 4]' in self.body
+        Spy.assert_ok(self.body, 'page_count', 2)
+        Spy.assert_ok(self.body, 'limit', 5)
+        Spy.assert_ok(self.body, 'pages', xrange(1, 3))
+
     def test_max_pages(self):
         self.request("/default_max_pages")
         assert '"data": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]' in self.body
         Spy.assert_ok(self.body, 'pages', xrange(1, 6))
 
-        self.request("/default_max_pages/?data_tgp_no=5")
+        self.request("/default_max_pages?data_tgp_no=5")
         assert '"data": [40, 41, 42, 43, 44, 45, 46, 47, 48, 49]' in self.body
         Spy.assert_ok(self.body, 'pages', xrange(3, 8))
 
@@ -433,7 +483,7 @@ class TestBasicPagination(TestPagination):
         assert '"data": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]' in self.body
         Spy.assert_ok(self.body, 'pages', xrange(1, 4))
 
-        self.request("/three_max_pages/?data_tgp_no=5")
+        self.request("/three_max_pages?data_tgp_no=5")
         assert '"data": [40, 41, 42, 43, 44, 45, 46, 47, 48, 49]' in self.body
         Spy.assert_ok(self.body, 'pages', xrange(4, 7))
 
@@ -442,20 +492,60 @@ class TestBasicPagination(TestPagination):
         assert '"data": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]' in self.body
         Spy.assert_ok(self.body, 'pages', xrange(1, 5))
 
-        self.request("/four_max_pages/?data_tgp_no=5")
+        self.request("/four_max_pages?data_tgp_no=5")
         assert '"data": [40, 41, 42, 43, 44, 45, 46, 47, 48, 49]' in self.body
         Spy.assert_ok(self.body, 'pages', xrange(4, 8))
 
+    def test_strict_parameters(self):
+        config.update({'tg.strict_parameters': True})
+        try:
+            # old style
+            self.request("/basic?tg_paginate_no=2")
+            assert self.status.startswith('200')
+            assert '"data": [4, 5, 6, 7]' in self.body
+            Spy.assert_ok(self.body, 'current_page', 2)
+            self.request('/custom_limit?tg_paginate_limit=2')
+            assert self.status.startswith('200')
+            assert '"data": [0, 1]' in self.body
+            Spy.assert_ok(self.body, 'limit', 2)
+            self.request("/default_max_pages?tg_paginate_no=5")
+            assert self.status.startswith('200')
+            assert '"data": [40, 41, 42, ' in self.body
+            Spy.assert_ok(self.body, 'pages', xrange(3, 8))
+            self.request("/basic?tg_paginate_rubbish=2")
+            assert self.status.startswith('500')
+            assert ("basic() got an unexpected keyword argument"
+                " 'tg_paginate_rubbish'") in self.body
+            # new style
+            self.request("/basic?data_tgp_no=2")
+            assert self.status.startswith('200')
+            assert '"data": [4, 5, 6, 7]' in self.body
+            Spy.assert_ok(self.body, 'current_page', 2)
+            self.request('/custom_limit?data_tgp_limit=2')
+            assert self.status.startswith('200')
+            assert '"data": [0, 1]' in self.body
+            Spy.assert_ok(self.body, 'limit', 2)
+            self.request("/default_max_pages?data_tgp_no=5")
+            assert self.status.startswith('200')
+            assert '"data": [40, 41, 42, ' in self.body
+            Spy.assert_ok(self.body, 'pages', xrange(3, 8))
+            self.request("/basic?data_tgp_rubbish=2")
+            assert self.status.startswith('500')
+            assert ("basic() got an unexpected keyword argument"
+                " 'data_tgp_rubbish'") in self.body
+        finally:
+            config.update({'tg.strict_parameters': False})
+
     def test_invalid_dynamic_limit(self):
         self.request("/invalid_dynamic")
-        assert cherrypy.response.status.startswith("500")
+        assert self.status.startswith("500")
         assert 'paginate: dynamic_limit (foobar) not found in output dict' in self.body
 
     def test_dynamic_limit(self):
         self.request("/dynamic")
         assert '"data": [0, 1, 2, 3, 4, 5, 6]' in self.body
 
-        self.request('/dynamic/?tg_paginate_limit=2')
+        self.request('/dynamic?tg_paginate_limit=2')
         assert '"data": [0, 1, 2, 3, 4, 5, 6]' in self.body
 
     def test_multiple(self):
@@ -463,73 +553,73 @@ class TestBasicPagination(TestPagination):
         assert '"foo": [0, 1, 2, 3]' in self.body
         assert '"bar": ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J"]' in self.body
 
-        self.request("/multiple/?foo_tgp_no=3")
+        self.request("/multiple?foo_tgp_no=3")
         assert '"foo": [8, 9, 10, 11]' in self.body
         assert '"bar": ["A", "B", "C", "D", "E", "F", "G", "H", "I", "J"]' in self.body
 
-        self.request("/multiple/?bar_tgp_no=2")
+        self.request("/multiple?bar_tgp_no=2")
         assert '"foo": [0, 1, 2, 3]' in self.body
         assert '"bar": ["K", "L", "M", "N", "O", "P", "Q", "R", "S", "T"]' in self.body
 
-        self.request("/multiple/?foo_tgp_no=2&bar_tgp_no=3")
+        self.request("/multiple?foo_tgp_no=2&bar_tgp_no=3")
         assert '"foo": [4, 5, 6, 7]' in self.body
         assert '"bar": ["U", "V", "W", "X", "Y", "Z"]' in self.body
 
     def test_out_of_bound_pages(self):
         for number in range(-3,1):
-            self.request("/basic/?tg_paginate_no=%s" % number)
+            self.request("/basic?tg_paginate_no=%s" % number)
             assert '"data": [0, 1, 2, 3]' in self.body
             Spy.assert_ok(self.body, 'current_page', 1)
             Spy.assert_ok(self.body, 'first_item', 1)
             Spy.assert_ok(self.body, 'last_item', 4)
 
         for number in range(4, 7):
-            self.request("/basic/?tg_paginate_no=%s" % number)
+            self.request("/basic?tg_paginate_no=%s" % number)
             assert '"data": [8, 9]' in self.body
             Spy.assert_ok(self.body, 'current_page', 3)
             Spy.assert_ok(self.body, 'first_item', 9)
             Spy.assert_ok(self.body, 'last_item', 10)
 
     def test_last_page(self):
-        self.request("/basic/?tg_paginate_no=last")
+        self.request("/basic?tg_paginate_no=last")
         assert '"data": [8, 9]' in self.body
         Spy.assert_ok(self.body, 'current_page', 3)
         Spy.assert_ok(self.body, 'first_item', 9)
         Spy.assert_ok(self.body, 'last_item', 10)
 
     def test_href(self):
-        self.request("/basic/?data_tgp_no=0") # out of bound
+        self.request("/basic?data_tgp_no=0") # out of bound
         Spy.assert_ok(self.body, 'current_page', 1)
         Spy.assert_ok(self.body, 'href_first', None)
         Spy.assert_ok(self.body, 'href_prev', None)
-        Spy.assert_ok(self.body, 'href_next', r"'/basic/?data_tgp_no=2&data_tgp_limit=4'", raw=True)
-        Spy.assert_ok(self.body, 'href_last', r"'/basic/?data_tgp_no=last&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_next', r"'/basic?data_tgp_no=2&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_last', r"'/basic?data_tgp_no=last&data_tgp_limit=4'", raw=True)
 
-        self.request("/basic/?data_tgp_no=1")
+        self.request("/basic?data_tgp_no=1")
         Spy.assert_ok(self.body, 'current_page', 1)
         Spy.assert_ok(self.body, 'href_first', None)
         Spy.assert_ok(self.body, 'href_prev', None)
-        Spy.assert_ok(self.body, 'href_next', r"'/basic/?data_tgp_no=2&data_tgp_limit=4'", raw=True)
-        Spy.assert_ok(self.body, 'href_last', r"'/basic/?data_tgp_no=last&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_next', r"'/basic?data_tgp_no=2&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_last', r"'/basic?data_tgp_no=last&data_tgp_limit=4'", raw=True)
 
-        self.request("/basic/?data_tgp_no=2")
+        self.request("/basic?data_tgp_no=2")
         Spy.assert_ok(self.body, 'current_page', 2)
-        Spy.assert_ok(self.body, 'href_first', r"'/basic/?data_tgp_no=1&data_tgp_limit=4'", raw=True)
-        Spy.assert_ok(self.body, 'href_prev', r"'/basic/?data_tgp_no=1&data_tgp_limit=4'", raw=True)
-        Spy.assert_ok(self.body, 'href_next', r"'/basic/?data_tgp_no=3&data_tgp_limit=4'", raw=True)
-        Spy.assert_ok(self.body, 'href_last', r"'/basic/?data_tgp_no=last&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_first', r"'/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_prev', r"'/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_next', r"'/basic?data_tgp_no=3&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_last', r"'/basic?data_tgp_no=last&data_tgp_limit=4'", raw=True)
 
-        self.request("/basic/?data_tgp_no=3")
+        self.request("/basic?data_tgp_no=3")
         Spy.assert_ok(self.body, 'current_page', 3)
-        Spy.assert_ok(self.body, 'href_first', r"'/basic/?data_tgp_no=1&data_tgp_limit=4'", raw=True)
-        Spy.assert_ok(self.body, 'href_prev', r"'/basic/?data_tgp_no=2&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_first', r"'/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_prev', r"'/basic?data_tgp_no=2&data_tgp_limit=4'", raw=True)
         Spy.assert_ok(self.body, 'href_next', None)
         Spy.assert_ok(self.body, 'href_last', None)
 
-        self.request("/basic/?data_tgp_no=4") # out of bound!
+        self.request("/basic?data_tgp_no=4") # out of bound!
         Spy.assert_ok(self.body, 'current_page', 3)
-        Spy.assert_ok(self.body, 'href_first', r"'/basic/?data_tgp_no=1&data_tgp_limit=4'", raw=True)
-        Spy.assert_ok(self.body, 'href_prev', r"'/basic/?data_tgp_no=2&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_first', r"'/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
+        Spy.assert_ok(self.body, 'href_prev', r"'/basic?data_tgp_no=2&data_tgp_limit=4'", raw=True)
         Spy.assert_ok(self.body, 'href_next', None)
         Spy.assert_ok(self.body, 'href_last', None)
 
@@ -541,20 +631,36 @@ class TestBasicPagination(TestPagination):
         Spy.assert_ok(self.body, 'href_next', None)
         Spy.assert_ok(self.body, 'href_last', None)
 
+    def test_with_webpath(self):
+        config.update({'server.webpath': '/web/path'})
+        try:
+            self.request("/web/path/basic?data_tgp_no=2")
+            Spy.assert_ok(self.body, 'current_page', 2)
+            Spy.assert_ok(self.body, 'href_first',
+                r"'/web/path/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
+            Spy.assert_ok(self.body, 'href_prev',
+                r"'/web/path/basic?data_tgp_no=1&data_tgp_limit=4'", raw=True)
+            Spy.assert_ok(self.body, 'href_next',
+                r"'/web/path/basic?data_tgp_no=3&data_tgp_limit=4'", raw=True)
+            Spy.assert_ok(self.body, 'href_last',
+                r"'/web/path/basic?data_tgp_no=last&data_tgp_limit=4'", raw=True)
+        finally:
+            config.update({'server.webpath': ''})
+
     def test_empty_data(self):
         self.request("/empty")
         assert '"data": []' in self.body
         Spy.assert_ok(self.body, 'limit', 4)
 
     def test_zero_limit(self):
-        self.request("/custom_limit/?data_tgp_limit=0")
+        self.request("/custom_limit?data_tgp_limit=0")
         assert '"data": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]' in self.body
         Spy.assert_ok(self.body, 'page_count', 1)
         Spy.assert_ok(self.body, 'limit', 10)
         Spy.assert_ok(self.body, 'pages', xrange(1, 2))
 
     def test_empty_data_zero_limit(self):
-        self.request("/empty/?data_tgp_limit=0")
+        self.request("/empty?data_tgp_limit=0")
         assert '"data": []' in self.body
         Spy.assert_ok(self.body, 'page_count', 0)
         Spy.assert_ok(self.body, 'limit', 1)
@@ -761,9 +867,6 @@ class TestDatabasePagination(TestPagination):
             return dict(data=data, spy=spy)
 
 
-    def setUp(self):
-        cherrypy.root = self.MyRoot()
-
     def assert_order(self, *args):
         expr = 'data="%s"' % ''.join(['[Address %r]' % x for x in args])
         if expr not in self.body:
@@ -773,7 +876,7 @@ class TestDatabasePagination(TestPagination):
     def test_basic(self):
         for test in "basic1", "basic2":
             for method in query_methods:
-                self.request("/%s/?method=%s" % (test, method))
+                self.request("/%s?method=%s" % (test, method))
                 self.assert_order(1, 2, 3, 4, 5, 6, 7, 8, 9, 10)
                 Spy.assert_ok(self.body, 'current_page', 1)
                 Spy.assert_ok(self.body, 'first_item', 1)
@@ -781,7 +884,7 @@ class TestDatabasePagination(TestPagination):
                 Spy.assert_ok(self.body, 'ordering', ['id'])
 
                 # old style
-                self.request("/%s/?method=%s&tg_paginate_no=2" % (test, method))
+                self.request("/%s?method=%s&tg_paginate_no=2" % (test, method))
                 self.assert_order(11, 12, 13, 14, 15, 16)
                 Spy.assert_ok(self.body, 'current_page', 2)
                 Spy.assert_ok(self.body, 'first_item', 11)
@@ -789,7 +892,7 @@ class TestDatabasePagination(TestPagination):
                 Spy.assert_ok(self.body, 'ordering', ['id'])
 
                 # new style
-                self.request("/%s/?method=%s&data_tgp_no=2" % (test, method))
+                self.request("/%s?method=%s&data_tgp_no=2" % (test, method))
                 self.assert_order(11, 12, 13, 14, 15, 16)
                 Spy.assert_ok(self.body, 'current_page', 2)
                 Spy.assert_ok(self.body, 'first_item', 11)
@@ -799,7 +902,7 @@ class TestDatabasePagination(TestPagination):
     def test_ordering(self):
         for test in "basic1", "basic2":
             for method in query_methods:
-                self.request("/%s/?method=%s&tg_paginate_ordering=%s" % \
+                self.request("/%s?method=%s&tg_paginate_ordering=%s" % \
                     (test, method, 'street'))
                 self.assert_order(16, 2, 3, 4, 5, 6, 7, 8, 9, 10)
                 Spy.assert_ok(self.body, 'current_page', 1)
@@ -807,7 +910,7 @@ class TestDatabasePagination(TestPagination):
                 Spy.assert_ok(self.body, 'last_item', 10)
                 Spy.assert_ok(self.body, 'ordering', ['street'])
 
-                self.request("/%s/?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s" % \
+                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s" % \
                     (test, method, 'street'))
                 self.assert_order(11, 12, 13, 14, 15, 1)
                 Spy.assert_ok(self.body, 'current_page', 2)
@@ -815,17 +918,32 @@ class TestDatabasePagination(TestPagination):
                 Spy.assert_ok(self.body, 'last_item', 16)
                 Spy.assert_ok(self.body, 'ordering', ['street'])
 
+    def test_strict_parameters(self):
+        config.update({'tg.strict_parameters': True})
+        try:
+            # old style
+            self.request("/basic1?method=Q&tg_paginate_ordering=street")
+            self.assert_order(16, 2, 3, 4, 5, 6, 7, 8, 9, 10)
+            Spy.assert_ok(self.body, 'ordering', ['street'])
+            # new style
+            self.request("/basic1?method=Q&data_tgp_ordering=street")
+            assert self.status.startswith('200')
+            self.assert_order(16, 2, 3, 4, 5, 6, 7, 8, 9, 10)
+            Spy.assert_ok(self.body, 'ordering', ['street'])
+        finally:
+            config.update({'tg.strict_parameters': False})
+
     def test_default_reversed(self):
         for test in "reversed1", "reversed2", "reversed3":
             for method in query_methods:
-                self.request("/%s/?method=%s" % (test, method))
+                self.request("/%s?method=%s" % (test, method))
                 self.assert_order(16, 15, 14, 13, 12, 11, 10, 9, 8, 7)
                 Spy.assert_ok(self.body, 'current_page', 1)
                 Spy.assert_ok(self.body, 'first_item', 1)
                 Spy.assert_ok(self.body, 'last_item', 10)
                 Spy.assert_ok(self.body, 'ordering', ['-id'])
 
-                self.request("/%s/?method=%s&data_tgp_no=2" % (test, method))
+                self.request("/%s?method=%s&data_tgp_no=2" % (test, method))
                 self.assert_order(6, 5, 4, 3, 2, 1)
                 Spy.assert_ok(self.body, 'current_page', 2)
                 Spy.assert_ok(self.body, 'first_item', 11)
@@ -834,20 +952,20 @@ class TestDatabasePagination(TestPagination):
 
     def test_invalid_default_reversed(self):
         for method in query_methods:
-            self.request("/wrong_reversed/?method=%s" % method)
-            assert cherrypy.response.status.startswith("500")
+            self.request("/wrong_reversed?method=%s" % method)
+            assert self.status.startswith("500")
             assert ('paginate: default_reversed (deprecated) only allowed'
                 ' when default_order is a basestring') in self.body
 
     def test_reverse_ordering(self):
         for test in "basic1", "basic2":
             for method in query_methods:
-                self.request("/%s/?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                     % (test, method, '-street'))
                 self.assert_order(1, 15, 14, 13, 12, 11, 10, 9, 8, 7)
                 Spy.assert_ok(self.body, 'ordering', ['-street'])
 
-                self.request("/%s/?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                     % (test, method, '-street'))
                 self.assert_order(6, 5, 4, 3, 2, 16)
                 Spy.assert_ok(self.body, 'ordering', ['-street'])
@@ -856,45 +974,45 @@ class TestDatabasePagination(TestPagination):
         for test in "basic1", "basic2":
             for method in query_methods:
                 # +/+
-                self.request("/%s/?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                     % (test, method, quote('city,street')))
                 self.assert_order(16, 3, 9, 10, 12, 13, 1, 4, 7, 8)
                 Spy.assert_ok(self.body, 'ordering', ['city', 'street'])
 
-                self.request("/%s/?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                     % (test, method, quote('city,street')))
                 self.assert_order(14, 15, 2, 5, 6, 11)
                 Spy.assert_ok(self.body, 'ordering', ['city', 'street'])
 
                 # +/-
-                self.request("/%s/?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                     % (test, method, quote('city,-street')))
                 self.assert_order(1, 13, 12, 10, 9, 3, 16, 15, 14, 8)
                 Spy.assert_ok(self.body, 'ordering', ['city', '-street'])
 
-                self.request("/%s/?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                     % (test, method, quote('city,-street')))
                 self.assert_order(7, 4, 11, 6, 5, 2)
                 Spy.assert_ok(self.body, 'ordering', ['city', '-street'])
 
                 # -/+
-                self.request("/%s/?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                     % (test, method, quote('-city,street')))
                 self.assert_order(2, 5, 6, 11, 4, 7, 8, 14, 15, 16)
                 Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])
 
-                self.request("/%s/?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                     % (test, method, quote('-city,street')))
                 self.assert_order(3, 9, 10, 12, 13, 1)
                 Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])
 
                 # -/-
-                self.request("/%s/?method=%s&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_ordering=%s"
                     % (test, method, quote('-city,-street')))
                 self.assert_order(11, 6, 5, 2, 15, 14, 8, 7, 4, 1)
                 Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])
 
-                self.request("/%s/?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
+                self.request("/%s?method=%s&tg_paginate_no=2&tg_paginate_ordering=%s"
                     % (test, method, quote('-city,-street')))
                 self.assert_order(13, 12, 10, 9, 3, 16)
                 Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])
@@ -902,136 +1020,136 @@ class TestDatabasePagination(TestPagination):
     def test_default_compound_ordering_1(self):
         # +/+
         for method in query_methods:
-            self.request("/default_compound_ordering1/?method=%s" % method)
+            self.request("/default_compound_ordering1?method=%s" % method)
             self.assert_order(16, 3, 9, 10, 12, 13, 1, 4, 7, 8)
             Spy.assert_ok(self.body, 'ordering', ['city', 'street'])
 
-            self.request("/default_compound_ordering1/?method=%s&tg_paginate_no=2" % method)
+            self.request("/default_compound_ordering1?method=%s&tg_paginate_no=2" % method)
             self.assert_order(14, 15, 2, 5, 6, 11)
             Spy.assert_ok(self.body, 'ordering', ['city', 'street'])
 
     def test_default_compound_ordering_2(self):
         # +/-
         for method in query_methods:
-            self.request("/default_compound_ordering2/?method=%s" % method)
+            self.request("/default_compound_ordering2?method=%s" % method)
             self.assert_order(1, 13, 12, 10, 9, 3, 16, 15, 14, 8)
             Spy.assert_ok(self.body, 'ordering', ['city', '-street'])
 
-            self.request("/default_compound_ordering2/?method=%s&tg_paginate_no=2" % method)
+            self.request("/default_compound_ordering2?method=%s&tg_paginate_no=2" % method)
             self.assert_order(7, 4, 11, 6, 5, 2)
             Spy.assert_ok(self.body, 'ordering', ['city', '-street'])
 
     def test_default_compound_ordering_3(self):
         # -/+
         for method in query_methods:
-            self.request("/default_compound_ordering3/?method=%s" % method)
+            self.request("/default_compound_ordering3?method=%s" % method)
             self.assert_order(2, 5, 6, 11, 4, 7, 8, 14, 15, 16)
             Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])
 
-            self.request("/default_compound_ordering3/?method=%s&tg_paginate_no=2" % method)
+            self.request("/default_compound_ordering3?method=%s&tg_paginate_no=2" % method)
             self.assert_order(3, 9, 10, 12, 13, 1)
             Spy.assert_ok(self.body, 'ordering', ['-city', 'street'])
 
     def test_default_compound_ordering_4(self):
         # -/-
         for method in query_methods:
-            self.request("/default_compound_ordering4/?method=%s" % method)
+            self.request("/default_compound_ordering4?method=%s" % method)
             self.assert_order(11, 6, 5, 2, 15, 14, 8, 7, 4, 1)
             Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])
 
-            self.request("/default_compound_ordering4/?method=%s&tg_paginate_no=2" % method)
+            self.request("/default_compound_ordering4?method=%s&tg_paginate_no=2" % method)
             self.assert_order(13, 12, 10, 9, 3, 16)
             Spy.assert_ok(self.body, 'ordering', ['-city', '-street'])
 
     def test_related_objects_ordering_level_1(self):
         # +/+
-        self.request("/related/?tg_paginate_ordering=%s" % quote('user.name,id'))
+        self.request("/related?tg_paginate_ordering=%s" % quote('user.name,id'))
         self.assert_order(1, 5, 3, 13, 7, 11, 16, 2, 12, 14)
         Spy.assert_ok(self.body, 'ordering', ['user.name', 'id'])
 
-        self.request("/related/?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
             % quote('user.name,id'))
         self.assert_order(10, 4, 6, 9, 15, 8)
         Spy.assert_ok(self.body, 'ordering', ['user.name', 'id'])
 
         # +/-
-        self.request("/related/?tg_paginate_ordering=%s" % quote('user.name,-id'))
+        self.request("/related?tg_paginate_ordering=%s" % quote('user.name,-id'))
         self.assert_order(5, 1, 3, 13, 16, 11, 7, 12, 2, 14)
         Spy.assert_ok(self.body, 'ordering', ['user.name', '-id'])
 
-        self.request("/related/?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
             % quote('user.name,-id'))
         self.assert_order(10, 15, 9, 6, 4, 8)
         Spy.assert_ok(self.body, 'ordering', ['user.name', '-id'])
 
         # -/+
-        self.request("/related/?tg_paginate_ordering=%s" % quote('-user.name,id'))
+        self.request("/related?tg_paginate_ordering=%s" % quote('-user.name,id'))
         self.assert_order(8, 4, 6, 9, 15, 10, 14, 2, 12, 7)
         Spy.assert_ok(self.body, 'ordering', ['-user.name', 'id'])
 
-        self.request("/related/?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
             % quote('-user.name,id'))
         self.assert_order(11, 16, 13, 3, 1, 5)
         Spy.assert_ok(self.body, 'ordering', ['-user.name', 'id'])
 
         # -/-
-        self.request("/related/?tg_paginate_ordering=%s" % quote('-user.name,-id'))
+        self.request("/related?tg_paginate_ordering=%s" % quote('-user.name,-id'))
         self.assert_order(8, 15, 9, 6, 4, 10, 14, 12, 2, 16)
         Spy.assert_ok(self.body, 'ordering', ['-user.name', '-id'])
 
-        self.request("/related/?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
             % quote('-user.name,-id'))
         self.assert_order(11, 7, 13, 3, 5, 1)
         Spy.assert_ok(self.body, 'ordering', ['-user.name', '-id'])
 
     def test_related_objects_ordering_level_2(self):
         # +/+
-        self.request("/related/?tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_ordering=%s"
             % quote('user.occupation.name,id'))
         self.assert_order(1, 5, 10, 13, 14, 2, 7, 11, 12, 16)
         Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', 'id'])
 
-        self.request("/related/?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
             % quote('user.occupation.name,id'))
         self.assert_order(4, 6, 8, 9, 15, 3)
         Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', 'id'])
 
         # +/-
-        self.request("/related/?tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_ordering=%s"
             % quote('user.occupation.name,-id'))
         self.assert_order(14, 13, 10, 5, 1, 16, 12, 11, 7, 2)
         Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', '-id'])
 
-        self.request("/related/?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
             % quote('user.occupation.name,-id'))
         self.assert_order(15, 9, 8, 6, 4, 3)
         Spy.assert_ok(self.body, 'ordering', ['user.occupation.name', '-id'])
 
         # -/+
-        self.request("/related/?tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_ordering=%s"
             % quote('-user.occupation.name,id'))
         self.assert_order(3, 4, 6, 8, 9, 15, 2, 7, 11, 12)
         Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', 'id'])
 
-        self.request("/related/?tg_paginate_no=2&tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"
             % quote('-user.occupation.name,id'))
         self.assert_order(16, 1, 5, 10, 13, 14)
         Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', 'id'])
 
         # -/-
-        self.request("/related/?tg_paginate_ordering=%s"
+        self.request("/related?tg_paginate_ordering=%s"
             % quote('-user.occupation.name,-id'))
         self.assert_order(3, 15, 9, 8, 6, 4, 16, 12, 11, 7)
         Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', '-id'])
 
-        self.request("/related/?tg_paginate_no=2&tg_paginate_ordering=%s"#
+        self.request("/related?tg_paginate_no=2&tg_paginate_ordering=%s"#
             % quote('-user.occupation.name,-id'))
         self.assert_order(2, 14, 13, 10, 5, 1)
         Spy.assert_ok(self.body, 'ordering', ['-user.occupation.name', '-id'])
 
     def test_zero_limit(self):
         for method in query_methods:
-            self.request("/zero_limit/?method=%s" % method)
+            self.request("/zero_limit?method=%s" % method)
             self.assert_order(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
 
     def test_ticket_1641(self):
