@@ -421,7 +421,13 @@ class TokenEater:
             self.__state = self.__waiting
 
     def __openseen(self, ttype, tstring, lineno):
-        if ttype == tokenize.OP and tstring == ')':
+        if ttype == tokenize.OP and tstring == ',':
+           # first handle case when the lang is explicitely set by the user
+           if self.__data:
+               self.__addentry(EMPTYSTRING.join(self.__data))
+           self.__state = self.__waiting
+
+        elif ttype == tokenize.OP and tstring == ')':
             # We've seen the last of the translatable strings.  Record the
             # line number of the first line of the strings and update the list
             # of messages seen.  Reset state for the next batch.  If there
@@ -429,8 +435,10 @@ class TokenEater:
             if self.__data:
                 self.__addentry(EMPTYSTRING.join(self.__data))
             self.__state = self.__waiting
+
         elif ttype == tokenize.STRING:
             self.__data.append(safe_eval(tstring))
+
         elif ttype not in [tokenize.COMMENT, token.INDENT, token.DEDENT,
                            token.NEWLINE, tokenize.NL]:
             # warn if we see anything else than STRING or whitespace
